@@ -24,8 +24,10 @@ formats.
 """
 
 import os
+import shutil
 import warnings
 import logging
+import time
 
 import numpy as np
 import zarr                                                 #pylint: disable=import-error
@@ -35,28 +37,38 @@ from PySide2.QtWidgets import QFileDialog, QMessageBox      #pylint: disable=no-
 from PySide2 import QtCore                                  #pylint: disable=import-error
 from PySide2.QtCore import Signal                           #pylint: disable=no-name-in-module
 
-from . import configuration as config
-from . ufsfile import UFSData
+import configuration as config                              #pylint: disable=import-error 
+from ufsfile import UFSData                                 #pylint: disable=import-error 
+from utils import clean_filename, now_string
 
-log = logging.getLogger(__name__)
-"Logger for this module."
+# Logger for this module. 
+_log = logging.getLogger(__name__)
 
-d = {}
-"Reference to the currently loaded data as a zarr array."
+#: Reference to the currently loaded data as a zarr group.
+d = zarr.group()
 
+#: Path to the currently loaded zarr data array in :attr:`d`.
 d_path = ""
-"Path to the currently loaded zarr data array in :attr:`d`."
 
+# Suffix to indicate temporary data stored on disk
+_temp_suffix = "-temporary.tr.zarr"
+
+#: Flag to indicate the currently loaded data is not saved to disk. It might be exclusively in RAM
+#: or on disk as a temporary data set. It's possible to have data in RAM that doesn't need saving
+#: though.
+d_unsaved = False
+
+#: Reference to temporary data which doesn't need to be saved to disk or can be rebuilt from :attr:`d`.
 t = zarr.group()
-"Reference to temporary data which doesn't need to be saved to disk or can be rebuilt from :attr:`d`."
 
+#: Attributes to include in the root of the zarr array.
 root_attrs = {
-    "Description"      : "Time-resolved spectroscopy data",
-    "Format"           : "trdata",
-    "Version"          : "1.0.0",
-    "CreationSoftware" : "trspectrometer",
+    "description"       : "Time-resolved spectroscopy data",
+    "format"            : "trdata",
+    "version"           : "1.0.0",
+    "creation_software" : "trspectrometer",
 }
-"Attributes to include in the root of the zarr array."
+
 
 class SignalStorage(QtCore.QObject):
     """
@@ -66,95 +78,106 @@ class SignalStorage(QtCore.QObject):
     is used to store and emit the various Signals.
     """
 
-    "Signal that new data has been loaded."
-    data_changed = Signal()
+    #: Signal that new data has been loaded.
+    #: If the parameter is ``True```, it indicates that a complete data set was loaded from a file,
+    #: and thus user interface controls should be updated to match. If ``False``, it indicates that
+    #: the new data is from an acquisition process, and UI updates should not be performed.
+    data_changed = Signal(bool)
+    
+    #: Signal that data has been updated.
+    #: The parameters are lists of scan, time, wavelength indices which may have changed.
+    #: None indicates that all elements have potentially changed.
+    data_updated = Signal(list, list, list)
 
-    "Signal that the selection of raw data traces has changed."
+    #: Signal that the selection of raw data traces has changed.
     raw_selection_changed = Signal()
+
 signals = SignalStorage()
 
 
-def null_raw_data():
+def new_zarr(location=""):
     """
-    Generate blank raw data.
+    Generate a new empty data set.
+
+    The directory location for the data files can be specified with the ``location`` parameter. If
+    ``location`` is an empty string (``""``), then an attempt at creating an automatically named
+    directory will be made. If the directory is unable to be created, or ``location=None``, then the
+    new data set will be stored in RAM instead and will be lost if not saved to disk at some later
+    stage.
+
+    Note that if the creation of the new data set succeeds, any existing unsaved data will be lost
+    without warnings.
+
+    The function will return the absolute path of the created data set, or an empty string if the
+    data set was created in RAM only.
+
+    :param location: Directory to use as storage for the new data set files.
+    :returns: Location on disk of the data set, or ``""`` if in RAM.
     """
-    data = zarr.group()
-    data.attrs.update(root_attrs)
-    data.create_group("raw")
-    data["raw"].array("wavelength", data=np.array([500.0], dtype=np.float32))
-    data["raw"].array("time", data=np.array([0.0], dtype=np.float32))
-    data["raw"].zeros("data", shape=(1, 1, 1), dtype=np.float32)
-    data["raw/wavelength"].attrs["units"] = "nm"
-    data["raw/time"].attrs["units"] = "ps"
-    data["raw/data"].attrs["units"] = "DeltaA"
-    return data
-
-def dummy_raw_data(scans=5, wl_size=2048, t_size=1500):
-    """
-    Generate some dummy raw data for testing purposes.
-
-    :param scans: Number of scans in the data set.
-    :param wl_size: Number if elements on the wavelength axis.
-    :param t_size: Number of elements on the time axis.
-    :returns: zarr group containing the dummy data.
-    """
-    data = zarr.group()
-    data.attrs.update(root_attrs)
-    data.create_group("raw")
-    data["raw"].array("wavelength", data=np.linspace(500.0, 1000.0, num=wl_size, dtype=np.float32))
-    data["raw"].array("time", data=np.linspace(0.0, 4000.0, num=t_size, dtype=np.float32))
-    data["raw"].zeros("data", shape=(0, t_size, wl_size), dtype=np.float32)
-    data["raw/wavelength"].attrs["units"] = "nm"
-    data["raw/time"].attrs["units"] = "ps"
-    data["raw/data"].attrs["units"] = "DeltaA"
-
-    # Add scans incrementally
-    for _ in range(scans):
-        scan = np.zeros((t_size, wl_size), dtype=np.float32)
-        # A spectral feature
-        scan[:,:] = norm.pdf(np.linspace(-5, 5, num=wl_size), loc=0, scale=1)[np.newaxis]
-        # With exponential decay kinetics
-        scan[:,:] *= np.exp(-(data["raw/time"][:]/1000))[:,np.newaxis]
-        scan[0:50] = 0.0
-        # Add some noise
-        scan += 0.1*(1 - 2*np.random.random_sample(size=(t_size, wl_size)))
-        # Add scan to data set
-        data["raw/data"].append(scan[np.newaxis], axis=0)
-
-    return data
-
-def raw_data_average():
-    """
-    Compute and return the average of the raw data set.
-
-    This routine will use the "exclude_scans" attribute of raw to exclude scans from the average.
-    The attribute should contain a list of scan indices to exclude.
-
-    :returns: Array containing the averaged data.
-    """
-    if (not "raw" in d) or (not "raw/data" in d): return None
-    if not "raw" in t:
-        t.create_group("raw")
-    if not "exclude_scans" in t["raw"].attrs:
-        # Include all scans if not specified yet
-        t["raw"].attrs["exclude_scans"] = []
-    if d["raw/data"].shape[0] - len(t["raw"].attrs["exclude_scans"]) >= 1:
-        # Multiple scans included, average them
-        selection_mask = np.ones((d["raw/data"].shape[0]), dtype=np.bool)
-        selection_mask[t["raw"].attrs["exclude_scans"]] = False
-        # Ignore warining if all NaNs in slice
-        warnings.simplefilter("ignore")
-        result = np.nanmean(d["raw/data"].get_orthogonal_selection((selection_mask, slice(None), slice(None))), axis=0)
-        warnings.simplefilter("default")
+    
+    global d, d_path, d_unsaved, t, _temp_suffix
+    
+    if location == "":
+        # Use auto generated name
+        location = os.path.abspath(os.path.join(config.data["directories"]["data"], f"{time.strftime('%y%m%d-%H%M%S', time.localtime())}{_temp_suffix}"))
+    elif location is not None:
+        # Some location was specified
+        if not os.path.isabs(location):
+            # Relative path, convert to absolute path from configured data directory
+            location = os.path.abspath(os.path.join(os.path.join(config.data["directories"]["data"], location)))
+        if not location.endswith(".tr.zarr"):
+            # Append suffix if needed
+            location = location + ".tr.zarr"
+    
+    if location:
+        # Create the data set on disk
+        if os.path.exists(location):
+            # Don't overwrite existing data
+            raise FileExistsError(f"Data location already exists: {location}")
+        if not os.access(os.path.dirname(location), os.W_OK):
+            # Can't write to location
+            raise PermissionError(f"Data location not writable: {location}")
+        data = zarr.group(store=location)
     else:
-        # Zero traces specified!?!? Just average everything again.
-        # TODO: Handle this better?
-        # Eventually we want to be able to handle completely missing raw data.
-        # Ignore warining if all NaNs in slice
-        warnings.simplefilter("ignore")
-        result = np.nanmean(d["raw/data"], axis=0)
-        warnings.simplefilter("default")
-    return result
+        # Location was specified as None, create in RAM
+        data = zarr.group()
+        location = ""
+    data.attrs.update(root_attrs)
+    data.attrs.update({"creation_time" : now_string()})
+
+    # Creation successful, start using the new data set
+    d = data
+    d_path = location
+    d_unsaved = False  # Nothing useful yet, no need to save
+    t = zarr.group()
+    _log.info("New data set created" + (f" at {location}" if location else "."))
+    return location
+
+
+def new_raw_zarr(location=""):
+    """
+    Generate a new data set containing some blank raw data.
+
+    The directory location for the data files can be specified with the ``location`` parameter. If
+    ``location`` is an empty string (``""``), then an attempt at creating an automatically named
+    directory will be made. If the directory is unable to be created, or ``location=None``, then the
+    new data set will be stored in RAM instead and will be lost if not saved to disk at some later
+    stage.
+
+    :param location: Directory to use as storage for the new data set files.
+    :returns: Location on disk of the data set, or ``""`` if in RAM.
+    """
+    global d, d_path, d_unsaved, t
+    new_zarr(location)
+    d.create_group("raw")
+    d["raw"].array("wavelength", data=np.array([500.0], dtype=np.float32))
+    d["raw"].array("time", data=np.array([0.0], dtype=np.float32))
+    d["raw"].full("data", np.nan, shape=(0, 1, 1), dtype=np.float32)
+    d["raw/wavelength"].attrs["units"] = config.data["rawdata"]["units"]["wavelength"]
+    d["raw/time"].attrs["units"] = config.data["rawdata"]["units"]["time"]
+    d["raw/data"].attrs["units"] = config.data["rawdata"]["units"]["data"]
+    return d_path
+
 
 def open_zarr(parent=None, initial_dir=None):
     """
@@ -169,7 +192,7 @@ def open_zarr(parent=None, initial_dir=None):
         If `None`, then the data directory specified in the configuration will be used.
     :returns: Name of selected data directory, or ``None`` if no directory selected.
     """
-    global d, t, d_path
+    global d, t, d_path, d_unsaved
     if initial_dir is None:
         initial_dir = config.data["directories"]["data"]
     # Was hoping that using a filter and directory options would limit to 
@@ -181,23 +204,27 @@ def open_zarr(parent=None, initial_dir=None):
     if qfd.exec_():
         dirname = qfd.selectedFiles()[0]
         try:
-            # Open read-only to prevent inadvertant modifications to data
+            # Open read-only to prevent inadvertent modifications to data
             data = zarr.open(dirname, mode="r")
         except:
             raise RuntimeError(f"Unable to read {dirname}:\nNot a Zarr data directory.")
-        if not "Format" in data.attrs or not data.attrs["Format"] == "trdata":
-            raise RuntimeError(f"Unable to read {dirname}:\nNot time-resolved data.")
+        if not "format" in data.attrs or not data.attrs["format"] == "trdata":
+            # Originally had capitalised "Format" field, keep compatible with that
+            if not "Format" in data.attrs or not data.attrs["Format"] == "trdata":
+                raise RuntimeError(f"Unable to read {dirname}:\nNot time-resolved data.")
         # Looks OK, switch in the loaded data
         d = data
         t = zarr.group()
-        # Copy data and metadata which may be modified to temporary storage
+        # Copy data and which may be modified to temporary storage
         if ("raw" in d) and ("exclude_scans" in d["raw"].attrs):
             t.create_group("raw")
             t["raw"].attrs["exclude_scans"] = d["raw"].attrs["exclude_scans"]
         d_path = dirname
+        d_unsaved = False
         return dirname
     return None
 
+
 def save_zarr(parent=None, initial_dir=None, data=None):
     """
     Show a dialog for selecting the save directory location for time-resolved data.
@@ -212,7 +239,7 @@ def save_zarr(parent=None, initial_dir=None, data=None):
     :returns: Directory where data was saved, or ``None`` if no directory selected.
     """
     # If data not given, default to currently loaded data
-    global d, t, d_path
+    global d, t, d_path, d_unsaved, _temp_suffix
     if data is None:
         if d is None: return None
         data = d
@@ -222,12 +249,32 @@ def save_zarr(parent=None, initial_dir=None, data=None):
     qfd.setAcceptMode(QFileDialog.AcceptSave)
     qfd.setFileMode(QFileDialog.Directory)
     qfd.setOption(QFileDialog.ShowDirsOnly)
+    # Suggest a file name based on stored metadata fields
+    datecode = time.strftime("%y%m%d_%H%M%S", time.localtime())
+    # Use updated metadata fields stored in temporary data storage if they exist
+    try:
+        if "sample" in t["raw"].attrs:
+            samplename = t["raw"].attrs["sample"]
+        else:
+            samplename = data["raw"].attrs["sample"]
+        if samplename: samplename = "-" + samplename
+    except:
+        samplename = ""
+    try:
+        if "pump" in t["raw"].attrs:
+            pumpname = t["raw"].attrs["pump"]
+        else:
+            pumpname = data["raw"].attrs["pump"]
+        if pumpname: pumpname = "-" + pumpname
+    except:
+        pumpname = ""
+    qfd.selectFile(clean_filename(f"{datecode}{samplename}{pumpname}.tr.zarr"))
     dirname = None
     while qfd.exec_():
         dirname = qfd.selectedFiles()[0]
         if dirname == d_path:
             # Selected data dir is the one currently open
-            reply = QMessageBox.question(parent, "Data Directory Exists", "Selected data directory is the one currently open. Do you want to overwrite its contents?")
+            reply = QMessageBox.question(parent, "TRSpectrometer", "Selected data directory is the one currently open. Do you want to overwrite its contents?")
             if reply == QMessageBox.No:
                 # Show selection dialog again
                 dirname = None
@@ -244,36 +291,147 @@ def save_zarr(parent=None, initial_dir=None, data=None):
                 pass
             # Ensure name has a .tr.zarr suffix
             dirname = dirname.partition(".tr.zarr")[0] + ".tr.zarr"
-            try:
-                dest = zarr.open(dirname, mode="w-")
-            except Exception as ex:
-                # Failed with "w-" mode as directory already exists
-                reply = QMessageBox.question(parent, "Data Directory Exists", "Selected data directory already exists. Do you want to overwrite its contents?")
-                if reply == QMessageBox.No:
-                    # Show selection dialog again
+            # Check if currently using a temporary storage location
+            if d_path.endswith(_temp_suffix):
+                # Can just move the whole directory
+                try:
+                    shutil.move(d_path, dirname)
+                except:
+                    # Failed to move for whatever reason, directory could exist etc
+                    reply = QMessageBox.information(parent, "TRSpectrometer", "Unable to create data directory in the selected location. Please choose another location or name.")
                     initial_dir = dirname
                     dirname = None
                     continue
-                else:
-                    # Force overwrite of directory
-                    dest = zarr.open(dirname, mode="w")
-            # Copy the data over to new destination
-            try:
-                zarr.copy_all(data, dest)
-                # Root attributes don't get copied. Bug in zarr 2.5.0
-                dest.attrs.update(data.attrs)
-            except Exception as ex:
-                raise RuntimeError(f"Unable to write {dirname}:\n{ex}")
+                # Open the new location for writing
+                dest = zarr.open(dirname, mode="r+")
+            else:
+                # Data in RAM or non-temporary location on disk
+                try:
+                    dest = zarr.open(dirname, mode="w-")
+                except Exception as ex:
+                    # Failed with "w-" mode as directory already exists
+                    reply = QMessageBox.question(parent, "TRSpectrometer", "Selected data directory already exists. Do you want to overwrite its contents?")
+                    if reply == QMessageBox.No:
+                        # Show selection dialog again
+                        initial_dir = dirname
+                        dirname = None
+                        continue
+                    else:
+                        # Force overwrite of directory
+                        dest = zarr.open(dirname, mode="w")
+                # Copy the data over to new destination
+                try:
+                    zarr.copy_all(data, dest)
+                    # Root attributes don't get copied. Bug in zarr 2.5.0
+                    dest.attrs.update(data.attrs)
+                except Exception as ex:
+                    raise RuntimeError(f"Unable to write {dirname}:\n{ex}")
         # Save out data modifications from temporary storage
-        if ("raw" in t) and ("exclude_scans" in t["raw"].attrs):
-            if "raw" not in dest:
-                dest.create_group("raw")
-            dest["raw"].attrs["exclude_scans"] = t["raw"].attrs["exclude_scans"]
+        def update_attr(attr_name):
+            if ("raw" in t) and (attr_name in t["raw"].attrs):
+                if "raw" not in dest:
+                    dest.create_group("raw")
+                dest["raw"].attrs[attr_name] = t["raw"].attrs[attr_name]
+        for attr in ["exclude_scans", "sample", "pump", "operator", "note"]:
+            update_attr(attr)
         # Done. Start using new data directory in read-only mode
         d_path = dirname
+        d_unsaved = False
         d = zarr.open(dirname, "r")
         break
     return dirname
+
+
+def prompt_unsaved(parent=None, initial_dir=None):
+    """
+    Check if the current data set is unsaved and prompt to save or discard it.
+
+    The function will return ``True`` if the data was handled OK, or ``False`` if the user cancelled
+    the operation or something went wrong.
+
+    :param parent: The parent QWidget for the dialog box.
+    :param initial_dir: Initial directory for the dialog box.
+    :returns: ``True`` if data was handled successfully.
+    """
+    global d, d_path, d_unsaved, t, _temp_suffix
+    if not d_path:
+        # Data in RAM
+        if not d_unsaved:
+            # Unsaved flag not set, nothing to do
+            return True
+        # Save needed
+        reply = QMessageBox.question(parent, "TRSpectrometer",
+                        "The current data set has not been saved.\nWould you like to save the existing data, or discard it?",
+                        QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel, QMessageBox.Cancel)
+        if reply == QMessageBox.Save:
+            if save_zarr(parent, initial_dir):
+                # Saved OK, job done
+                return True
+        elif reply == QMessageBox.Discard:
+            # Allow data in RAM to be lost
+            return True
+        # Cancelled, or not saved properly
+        return False
+    elif d_path.endswith(_temp_suffix):
+        # Data in temporary disk storage, prompt for save (regardless of unsaved flag)
+        reply = QMessageBox.question(parent, "TRSpectrometer",
+                        f"The current data set is in temporary storage on disk at:\n\n{os.path.dirname(d_path)}{os.sep}\n{os.path.basename(d_path)}\n\n"
+                        "Would you like to save it elsewhere, discard it, or ignore it and leave the temporary files on disk?",
+                        QMessageBox.Save | QMessageBox.Discard | QMessageBox.Ignore | QMessageBox.Cancel, QMessageBox.Cancel)
+        if reply == QMessageBox.Save:
+            if save_zarr(parent, initial_dir):
+                # Saved OK, job done
+                return True
+        elif reply == QMessageBox.Discard:
+            # Delete the temporary files
+            d_path_old = d_path
+            # We'll make new data, in case there's issues deleting open files (on Windows...)
+            new_raw_zarr(location=None)
+            signals.data_changed.emit(False)
+            try:
+                shutil.rmtree(d_path_old)
+            except:
+                _log.warning(f"Unable to delete temporary data at {d_path_old}")
+                QMessageBox.warning(parent, "TRSpectrometer", f"An error occurred while removing temporary files at:\n{d_path_old}\nYou may wish to try deleting these files manually later.")
+            return True
+        elif reply == QMessageBox.Ignore:
+            # Ignore and leave data on disk, nothing to do
+            return True
+        # Cancelled, or not saved properly
+        return False
+    else:
+        # Data on disk, not in temporary storage
+        if not d_unsaved:
+            # Unsaved flag not set, nothing to do
+            return True
+        # Data modified, save needed
+        reply = QMessageBox.question(parent, "TRSpectrometer",
+                        "The current data set has been modifed but not yet saved.\nWould you like to apply the changes to the existing data, save a new data set, or discard the modifications?",
+                        QMessageBox.Apply | QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel, QMessageBox.Cancel)
+        if reply == QMessageBox.Save:
+            if save_zarr(parent, initial_dir):
+                # Saved OK, job done
+                return True
+        elif reply == QMessageBox.Discard:
+            # Allow changes to be lost
+            return True
+        elif reply == QMessageBox.Apply:
+            # Save temporary structures to disk
+            try:
+                # Enable write
+                d = zarr.open(d_path, mode="r+")
+                if ("raw" in t) and ("exclude_scans" in t["raw"].attrs):
+                    if "raw" not in d:
+                        d.create_group("raw")
+                    d["raw"].attrs["exclude_scans"] = t["raw"].attrs["exclude_scans"]
+                d_unsaved = False
+                d = zarr.open(d_path, "r")
+            except:
+                _log.warning(f"Unable to update data at {d_path}")
+                QMessageBox.warning(parent, "TRSpectrometer", f"An error occurred while updating the data at:\n{d_path}")
+        # Cancelled, or not saved properly
+        return False
+
         
 
 def import_raw(parent=None, initial_dir=None):
@@ -305,9 +463,9 @@ def import_raw(parent=None, initial_dir=None):
         data["raw"].array("wavelength", data=np.mean(data_matrix[:,0,1:], axis=0))
         data["raw"].array("time", data=np.mean(data_matrix[:,1:,0], axis=0))
         data["raw"].array("data", data=data_matrix[:,1:,1:])
-        data["raw/wavelength"].attrs["units"] = "nm"
-        data["raw/time"].attrs["units"] = "ps"
-        data["raw/data"].attrs["units"] = "DeltaA"        
+        data["raw/wavelength"].attrs["units"] = config.data['rawdata']['units']['wavelength']
+        data["raw/time"].attrs["units"] = config.data['rawdata']['units']['time']
+        data["raw/data"].attrs["units"] = config.data['rawdata']['units']['data']
     elif filetype == type_ufs:
         ufs = [ UFSData(ufsfile=f) for f in filenames ]
         # UFS files can potentially contain multiple scans.
@@ -404,10 +562,30 @@ def export_raw_average(parent=None, initial_dir=None):
     """
     if initial_dir is None:
         initial_dir = config.data["directories"]["data"]
+    # Suggest a file name based on stored metadata fields
+    datecode = time.strftime("%y%m%d_%H%M%S", time.localtime())
+    # Use updated metadata fields stored in temporary data storage if they exist
+    try:
+        if "sample" in t["raw"].attrs:
+            samplename = t["raw"].attrs["sample"]
+        else:
+            samplename = data["raw"].attrs["sample"]
+        if samplename: samplename = "-" + samplename
+    except:
+        samplename = ""
+    try:
+        if "pump" in t["raw"].attrs:
+            pumpname = t["raw"].attrs["pump"]
+        else:
+            pumpname = data["raw"].attrs["pump"]
+        if pumpname: pumpname = "-" + pumpname
+    except:
+        pumpname = ""
+    initial_filename = os.path.join(initial_dir, clean_filename(f"{datecode}{samplename}{pumpname}"))
     # File type definitions
     type_csv = "CSV Files (*.csv)"
     type_ufs = "UFS Files (*.ufs)"
-    filename, filetype = QFileDialog.getSaveFileName(parent, "Export Raw Data Average", initial_dir, f"{type_csv};;{type_ufs}")
+    filename, filetype = QFileDialog.getSaveFileName(parent, "Export Raw Data Average", initial_filename, f"{type_csv};;{type_ufs}")
     if not filename: return None
     if filetype == type_csv:
         # Ensure name has a .csv suffix
@@ -439,6 +617,38 @@ def export_raw_average(parent=None, initial_dir=None):
     return filename
 
 
+def raw_data_average():
+    """
+    Compute and return the average of the raw data set.
+
+    This routine will use the "exclude_scans" attribute of raw to exclude scans from the average.
+    The attribute should contain a list of scan indices to exclude.
+
+    :returns: Array containing the averaged data.
+    """
+    if (not "raw" in d) or (not "raw/data" in d): return None
+    if not "raw" in t:
+        t.create_group("raw")
+    if not "exclude_scans" in t["raw"].attrs:
+        # Include all scans if not specified yet
+        t["raw"].attrs["exclude_scans"] = []
+    if  0 < len(t["raw"].attrs["exclude_scans"]) < d["raw/data"].shape[0]:
+        # Multiple scans included, average them
+        selection_mask = np.ones((d["raw/data"].shape[0]), dtype=bool)
+        selection_mask[t["raw"].attrs["exclude_scans"]] = False
+        # Ignore warning if all NaNs in slice
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            result = np.nanmean(d["raw/data"].get_orthogonal_selection((selection_mask, slice(None), slice(None))), axis=0)
+    else:
+        # Zero or all scans selected
+        # Ignore warning if all NaNs in slice
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            result = np.nanmean(d["raw/data"], axis=0)
+    return result
+
+
 def find_stepsizes(time_axis, filter_window=35, snap=0.05):
     """
     Generate a list of time step sizes from a given time axis label array.
@@ -458,4 +668,4 @@ def find_stepsizes(time_axis, filter_window=35, snap=0.05):
         else:
             # New step size
             steplist.append([1, x])
-    return steplist
\ No newline at end of file
+    return steplist
